"
COPYRIGHT (c) 2020-2022 LabWare
"
"{ Package: 'stx:goodies/monticello/tonel' }"

"{ NameSpace: Smalltalk }"

MCDirectoryRepository subclass:#TonelRepository
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Monticello-Tonel-Core'
!

!TonelRepository class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020-2022 LabWare


"
! !

!TonelRepository class methodsFor:'instance creation'!

basicFromUrl: anUrl
	"remove 'tonel://' and go"
	^ self new directory: (anUrl asString allButFirst: self description size) asFileReference
! !

!TonelRepository class methodsFor:'accessing'!

description
    ^ 'tonel://'
!

urlSchemes
	^ #(tonel)
! !

!TonelRepository class methodsFor:'metacello support'!

createRepositoryFromSpec: aRepositorySpec on: aPlatform
	^ aPlatform createTonelRepository: aRepositorySpec
!

isAvailableFor: type
	^ type = 'tonel'
!

isFiletreeAlternateFormat: aRepositorySpec
	| location propRef |
	
	location := (aRepositorySpec description allButFirst: ('filetree://' size)) asFileReference.
	propRef := location / '.properties'.
	propRef exists ifFalse: [ ^ false ].
	
	^ propRef readStreamDo:[ :stream | 
		((STONJSON fromStream: stream) at: #format ifAbsent: []) = #tonel ]
! !

!TonelRepository class methodsFor:'utilities'!

discover: aStringOrFilename
    "Search for a Tonel repository in given directory (or super-directories)
     and return it (as an instance of Filename). If no repository is found, 
     returns nil.

     A non-nil return value can be directly used to create a TonelRepository
     as:

         | repoPath repo |

         repoPath := TonelRepository discover: '/some/path'.
         repoPath notNil ifTrue: [
            repo := TonelRepository directory: repoPath.            
         ].

     A Tonel repository is detected by searching for (STON) file .project
     If found, contents of #srcDirectory is consulted to create a repository
     directory.
    "

    | prjDir |

    prjDir := aStringOrFilename asFilename.
    (prjDir / '.project') exists ifTrue: [
        | prjProps |

        prjProps := (prjDir / '.project') readingFileDo: [: s | STON fromStream: s ].
        (prjProps includesKey: 'srcDirectory') ifTrue: [
            | srcDir |

            srcDir := prjDir / (prjProps at: 'srcDirectory').
            srcDir isDirectory ifTrue: [
                ^ srcDir
            ].
        ].
    ].
    prjDir isRootDirectory ifTrue: [
        ^ nil
    ].
    ^ self discover: prjDir directory

    "
    TonelRepository discover: '/home/jv/Projects/pharo'
    TonelRepository discover: '/home/jv/Projects/MachineArithmetic'
    TonelRepository discover: '/tmp'
    "

    "Created: / 31-08-2020 / 13:50:09 / Jan Vrany <jan.vrany@labware.com>"
!

discoverPackage: packageId
    "Find a Tonel repository in current package path for given package id.
     and return it (as an instance of Filename). If no repository is found, 
     returns nil.

     A non-nil return value can be directly used to create a TonelRepository
     as:

         | repoPath repo |

         repoPath := TonelRepository discover: '/some/path'.
         repoPath notNil ifTrue: [
            repo := TonelRepository directory: repoPath.            
         ]. 
    "

    Smalltalk packagePath do: [:each | 
        | repoPath |

        repoPath := self discover: each.
        repoPath notNil ifTrue: [
            | repo |

            repo := self directory: repoPath. 
            (repo allPackageNames includes: packageId) ifTrue: [
                ^ repoPath                
            ].
        ].
    ].
    ^ nil.

    "
    Smalltalk packagePath add: '/home/jv/Projects/MachineArithmetic'.
    TonelRepository discoverPackage: 'MachineArithmetic'
    TonelRepository discoverPackage: 'Bogus'     
    "
! !

!TonelRepository methodsFor:'accessing'!

allFileNames
        ^ super allFileNames 
                select: [ :each | 
                        self fileUtils 
                                filePathExists: each, '/package.st' 
                                relativeTo: self directory ]

    "Created: / 09-07-2020 / 23:03:29 / Jan Vrany <jan.vrany@labware.com>"
!

allFileNamesForVersionNamed: aString
	^ self 
		filterFileNames: self readableFileNames 
		forVersionNamed: aString
!

allPackageNames
    ^ self allFileNames

    "Created: / 09-07-2020 / 23:00:41 / Jan Vrany <jan.vrany@labware.com>"
!

asRepositorySpecFor: aMetacelloMCProject
    ^ aMetacelloMCProject repositorySpec
        description: self description;
        type: 'tonel';
        yourself
!

basicStoreVersion: aVersion
  TonelWriter 
	fileOut: aVersion 
	on: self directory
!

description
    ^ self class description, super description
!

fileDirectoryOn: directoryPath
	^ self fileUtils 
		directoryFromPath: directoryPath 
		relativeTo: self directory
!

fileUtils
	^ TonelFileUtils current
!

filterFileNames: aCollection forVersionNamed: aString
	^ aCollection select: [:ea | (self versionNameFromFileName: ea) = aString]
!

loadAllFileNames
	^ super loadAllFileNames 
		select: [ :each | 
			self fileUtils 
				filePathExists: each, '/package.st' 
				relativeTo: self directory ]
!

packageDescriptionFromPackageDirectory: packageDirectory
    | filename |
    filename := self fileUtils directoryName: packageDirectory.
    ^ {filename. 'tonel'. 1. filename}
!

packageDescriptionsFromReadableFileNames
    ^ ((self readableFileNames 
		collect: [ :each | self fileUtils directoryFromPath: each relativeTo: self directory ])
		select: [ :each | self fileUtils directoryExists: each ])
		collect: [ :each | self packageDescriptionFromPackageDirectory: each ]
!

readableFileNames
	| all cached new |

	all := self allFileNames. "in disk"
	all := all reject: [ :each | each beginsWith: '.' ].
	cached := self cachedFileNames. "in memory"
	new := all \ cached.
	^ (cached asArray, new) select: [:ea | self canReadFileNamed: ea]
!

readableFileReferences
	^ self readableFileNames collect: [ :each | 
		self fileUtils 
			directoryFromPath: each 
			relativeTo: self directory ]
!

versionFrom: aVersionReferenceString
    "until we no longer find .tree directories in the wild"

   	^ self readableFileReferences
		detect: [ :each | 
			(self fileUtils directoryExists: each)
			and: [ 
				each basename = aVersionReferenceString 
				or: [ (self versionInfoForPackageDirectory: each) name = aVersionReferenceString ] ] ]
		ifFound: [ :packageDirectory | 
			self loadVersionFromFileNamed: (self fileUtils directoryName: packageDirectory) ]
		ifNone: [ 
			nil ].
!

versionReaderForFileNamed: aString do: aBlock
    | reader |

    reader := TonelReader on: directory pathName fileName: aString.
    ^ aBlock value: reader

    "Created: / 09-07-2020 / 23:14:02 / Jan Vrany <jan.vrany@labware.com>"
! !

!TonelRepository methodsFor:'actions'!

readStreamForFileNamed: aString do: aBlock
    ^ aBlock value: self directory
! !

!TonelRepository methodsFor:'metacello support'!

goferVersionFrom: aVersionReference
	"Simillar hack than FileTree repositories."

	(self readableFileNames collect: [ :fileName | self fileDirectoryOn: fileName ])
		do: [ :packageDirectory |
			((self fileUtils directoryExists: packageDirectory )
			and: [(self versionInfoForPackageDirectory: packageDirectory) name = aVersionReference name])
				ifTrue: [ ^ self loadVersionFromFileNamed: (self fileUtils directoryName: packageDirectory) ] ].
	^nil
! !

!TonelRepository methodsFor:'versions'!

versionInfoForPackageDirectory: packageDirectory
	^ ((MCReader readerClassForFileNamed: (self fileUtils directoryName: packageDirectory))
		on: (self fileUtils parentDirectoryOf: packageDirectory)
		fileName: (self fileUtils directoryName: packageDirectory))
		loadVersionInfo;
		info
!

versionInfoFromVersionNamed: aString
	^ self versionInfoForPackageDirectory: (self fileDirectoryOn: aString)
!

versionNameFromFileName: aString
	| description |
	description := self packageDescriptionFromPackageDirectory: (self fileDirectoryOn: aString).
	^ description first , '-' , description second , '.' , description third printString
!

versionWithInfo: aVersionInfo ifAbsent: errorBlock
	(self allFileNamesForVersionNamed: aVersionInfo name)
		ifNotEmpty: [ :aCollection | ^ self versionFromFileNamed: aCollection first ].
	^ errorBlock value
! !

!TonelRepository class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

